@@ -196,29 +196,11 @@ namespace Banshee.MediaEngine
         private void HandleStateChange (PlayerEventStateChangeArgs args)
         {
             if (args.Current == PlayerState.Loaded && CurrentTrack != null) {
-                active_engine.Volume = (ushort) VolumeSchema.Get ();
                 MetadataService.Instance.Lookup (CurrentTrack);
             } else if (args.Current == PlayerState.Ready) {
                 // Enable our preferred equalizer if it exists and was enabled last time.
-                if (SupportsEqualizer && EqualizerSetting.EnabledSchema.Get ()) {
-                    string name = EqualizerSetting.PresetSchema.Get();
-                    if (!String.IsNullOrEmpty (name)) {
-                        // Don't use EqualizerManager.Instance - used by the eq dialog window.
-                        EqualizerManager manager = new EqualizerManager (EqualizerManager.Instance.Path);
-                        manager.Load ();
-                        EqualizerSetting equalizer = null;
-                        foreach (EqualizerSetting eq in manager) {
-                            if (eq.Name == name) {
-                                equalizer = eq;
-                                break;
-                            }
-                        }
-
-                        if (equalizer != null) {
-                            Log.DebugFormat ("Enabling equalizer preset: {0}", equalizer.Name);
-                            manager.Enable (equalizer);
-                        }
-                    }
+                if (SupportsEqualizer) {
+                    EqualizerManager.Instance.Select ();
                 }
 
                 if (pending_playback_for_not_ready != null) {
@@ -246,6 +228,10 @@ namespace Banshee.MediaEngine
                 }
             }
 
+            if (args.Event == PlayerEvent.StartOfStream) {
+                incremented_last_played = false;
+            }
+
             RaiseEvent (args);
 
             // Do not raise iterate across DBus to avoid so many calls;
@@ -303,6 +289,46 @@ namespace Banshee.MediaEngine
             OpenCheck (new SafeUri (uri));
         }
 
+        public void SetNextTrack (TrackInfo track)
+        {
+            if (track != null && EnsureActiveEngineCanPlay (track.Uri)) {
+                active_engine.SetNextTrack (track);
+            } else {
+                active_engine.SetNextTrack ((TrackInfo) null);
+            }
+        }
+
+        public void SetNextTrack (SafeUri uri)
+        {
+            if (EnsureActiveEngineCanPlay (uri)) {
+                active_engine.SetNextTrack (uri);
+            } else {
+                active_engine.SetNextTrack ((SafeUri) null);
+            }
+        }
+
+        private bool EnsureActiveEngineCanPlay (SafeUri uri)
+        {
+            if (uri == null) {
+                // No engine can play the null URI.
+                return false;
+            }
+            if (active_engine != FindSupportingEngine (uri)) {
+                if (active_engine.CurrentState == PlayerState.Playing) {
+                    // If we're currently playing then we can't switch engines now.
+                    // We can't ensure the active engine can play this URI.
+                    return false;
+                } else {
+                    // If we're not playing, we can switch the active engine to
+                    // something that will play this URI.
+                    SwitchToEngine (FindSupportingEngine (uri));
+                    CheckPending ();
+                    return true;
+                }
+            }
+            return true;
+        }
+
         public void OpenPlay (TrackInfo track)
         {
             OpenPlay (track, true);
@@ -349,17 +375,14 @@ namespace Banshee.MediaEngine
                 return;
             }
 
-            IncrementLastPlayed ();
-
-            FindSupportingEngine (uri);
+            PlayerEngine supportingEngine = FindSupportingEngine (uri);
+            SwitchToEngine (supportingEngine);
             CheckPending ();
 
             if (track != null) {
                 active_engine.Open (track);
-                incremented_last_played = false;
             } else if (uri != null) {
                 active_engine.Open (uri);
-                incremented_last_played = false;
             }
 
             if (play) {
@@ -367,48 +390,56 @@ namespace Banshee.MediaEngine
             }
         }
 
-        private bool incremented_last_played = true;
         public void IncrementLastPlayed ()
+        {
+            // If Length <= 0 assume 100% completion.
+            IncrementLastPlayed (active_engine.Length <= 0
+                ? 1.0
+                : (double)active_engine.Position / active_engine.Length);
+        }
+
+        private bool incremented_last_played = true;
+        public void IncrementLastPlayed (double completed)
         {
             if (!incremented_last_played && CurrentTrack != null && CurrentTrack.PlaybackError == StreamPlaybackError.None) {
-                //if Length <= 0 assume 100% completion:
-                if (active_engine.Length <= 0) {
-                    CurrentTrack.OnPlaybackFinished (1);
-                } else {
-                    CurrentTrack.OnPlaybackFinished ((double)active_engine.Position / (double)active_engine.Length);
-                }
+                CurrentTrack.OnPlaybackFinished (completed);
                 incremented_last_played = true;
             }
         }
 
-        private void FindSupportingEngine (SafeUri uri)
+        private PlayerEngine FindSupportingEngine (SafeUri uri)
         {
             foreach (PlayerEngine engine in engines) {
                 foreach (string extension in engine.ExplicitDecoderCapabilities) {
                     if (!uri.AbsoluteUri.EndsWith (extension)) {
                         continue;
-                    } else if (active_engine != engine) {
-                        Close ();
-                        pending_engine = engine;
-                        Log.DebugFormat ("Switching engine to: {0}", engine.GetType ());
                     }
-                    return;
+                    return engine;
                 }
             }
 
             foreach (PlayerEngine engine in engines) {
                 foreach (string scheme in engine.SourceCapabilities) {
                     bool supported = scheme == uri.Scheme;
-                    if (supported && active_engine != engine) {
-                        Close ();
-                        pending_engine = engine;
-                        Log.DebugFormat ("Switching engine to: {0}", engine.GetType ());
-                        return;
-                    } else if (supported) {
-                        return;
+                    if (supported) {
+                        return engine;
                     }
                 }
             }
+            // If none of our engines support this URI, return the currently active one.
+            // There doesn't seem to be anything better to do.
+            return active_engine;
+        }
+
+        private bool SwitchToEngine (PlayerEngine switchTo)
+        {
+            if (active_engine != switchTo) {
+                Close ();
+                pending_engine = switchTo;
+                Log.DebugFormat ("Switching engine to: {0}", switchTo.GetType ());
+                return true;
+            }
+            return false;
         }
 
         public void Close ()
@@ -475,6 +506,11 @@ namespace Banshee.MediaEngine
             active_engine.VideoExpose (displayContext, direct);
         }
 
+        public void VideoWindowRealize (IntPtr displayContext)
+        {
+            active_engine.VideoWindowRealize (displayContext);
+        }
+
         public IntPtr VideoDisplayContext {
             set { active_engine.VideoDisplayContext = value; }
             get { return active_engine.VideoDisplayContext; }
@@ -501,8 +537,8 @@ namespace Banshee.MediaEngine
 
         private void CheckPending ()
         {
-            if(pending_engine != null && pending_engine != active_engine) {
-                if(active_engine.CurrentState == PlayerState.Idle) {
+            if (pending_engine != null && pending_engine != active_engine) {
+                if (active_engine.CurrentState == PlayerState.Idle) {
                     Close ();
                 }
 
@@ -654,7 +690,8 @@ namespace Banshee.MediaEngine
             | PlayerEvent.Error
             | PlayerEvent.Volume
             | PlayerEvent.Metadata
-            | PlayerEvent.TrackInfoUpdated;
+            | PlayerEvent.TrackInfoUpdated
+            | PlayerEvent.RequestNextTrack;
 
         private const PlayerEvent event_default_mask = event_all_mask & ~PlayerEvent.Iterate;
 
