@@ -30,6 +30,8 @@ using System;
 using Gtk;
 using Mono.Unix;
 
+using Hyena;
+
 using Banshee.Base;
 using Banshee.ServiceStack;
 using Banshee.MediaEngine;
@@ -41,38 +43,58 @@ namespace Banshee.Gui
     public abstract class BaseClientWindow : Window
     {
         private PersistentWindowController window_controller;
+        private string accel_map_file = Paths.Combine (Paths.ApplicationData, "gtk_accel_map");
 
         private GtkElementsService elements_service;
         protected GtkElementsService ElementsService {
             get { return elements_service; }
         }
-        
+
         private InterfaceActionService action_service;
         protected InterfaceActionService ActionService {
             get { return action_service; }
         }
-        
+
         public event EventHandler TitleChanged;
 
         protected BaseClientWindow (IntPtr ptr) : base (ptr)
         {
         }
-    
+
         public BaseClientWindow (string title, string configNameSpace, int defaultWidth, int defaultHeight) : base (title)
         {
-            elements_service = ServiceManager.Get<GtkElementsService> ("GtkElementsService");
-            action_service = ServiceManager.Get<InterfaceActionService> ("InterfaceActionService");
-            
+            elements_service = ServiceManager.Get<GtkElementsService> ();
+            action_service = ServiceManager.Get<InterfaceActionService> ();
+
             ConfigureWindow ();
 
             window_controller = new PersistentWindowController (this, configNameSpace, defaultWidth, defaultHeight, WindowPersistOptions.All);
             window_controller.Restore ();
-            
+
             elements_service.PrimaryWindow = this;
-            
+
             AddAccelGroup (action_service.UIManager.AccelGroup);
-            
+
             InitializeWindow ();
+
+            try {
+                if (System.IO.File.Exists (accel_map_file)) {
+                    Gtk.AccelMap.Load (accel_map_file);
+                }
+            } catch (Exception e) {
+                Hyena.Log.Exception ("Failed to load custom AccelMap", e);
+            }
+        }
+
+        public override void Dispose ()
+        {
+            base.Dispose ();
+
+            try {
+                Gtk.AccelMap.Save (accel_map_file);
+            } catch (Exception e) {
+                Hyena.Log.Exception ("Failed to save custom AccelMap", e);
+            }
         }
 
 		protected void InitialShowPresent ()
@@ -90,7 +112,7 @@ namespace Banshee.Gui
 		}
 
         public virtual Box ViewContainer { get { return null; } }
-        
+
         public void ToggleVisibility ()
         {
             SetVisible (!(Visible && IsActive));
@@ -108,31 +130,31 @@ namespace Banshee.Gui
                 Present ();
             }
         }
-        
+
         private void InitializeWindow ()
         {
             Initialize ();
         }
-        
+
         protected abstract void Initialize ();
 
         protected virtual void ConnectEvents ()
         {
-            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent, 
+            ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent,
                 PlayerEvent.StartOfStream |
                 PlayerEvent.TrackInfoUpdated |
                 PlayerEvent.EndOfStream);
         }
 
-        private void OnPlayerEvent (PlayerEventArgs args) 
+        private void OnPlayerEvent (PlayerEventArgs args)
         {
             UpdateTitle ();
         }
-    
+
         protected virtual void ConfigureWindow ()
         {
         }
-    
+
         protected override bool OnDeleteEvent (Gdk.Event evnt)
         {
              if (ElementsService.PrimaryWindowClose != null) {
@@ -140,23 +162,25 @@ namespace Banshee.Gui
                     return true;
                 }
             }
-            
+
             Banshee.ServiceStack.Application.Shutdown ();
-            return base.OnDeleteEvent (evnt);
+            return true;
         }
 
         protected override bool OnWindowStateEvent (Gdk.EventWindowState evnt)
         {
-            ToggleAction fullscreen_action = (ToggleAction) ServiceManager.Get<InterfaceActionService> ().ViewActions["FullScreenAction"];
-            fullscreen_action.Active = (evnt.NewWindowState & Gdk.WindowState.Fullscreen) != 0;
-            
+            var action_service = ServiceManager.Get<InterfaceActionService> ();
+            if (action_service != null) {
+                ((ToggleAction)action_service.ViewActions["FullScreenAction"]).Active = (evnt.NewWindowState & Gdk.WindowState.Fullscreen) != 0;
+            }
+
             if ((evnt.NewWindowState & Gdk.WindowState.Withdrawn) == 0) {
                 window_controller.Save ();
             }
-            
+
             return base.OnWindowStateEvent (evnt);
         }
-        
+
         protected virtual void OnTitleChanged ()
         {
             EventHandler handler = TitleChanged;
@@ -164,19 +188,19 @@ namespace Banshee.Gui
                 handler (this, EventArgs.Empty);
             }
         }
-        
+
         protected virtual void UpdateTitle ()
         {
             TrackInfo track = ServiceManager.PlayerEngine.CurrentTrack;
             if (track != null) {
                 // Translators: this is the window title when a track is playing
                 //              {0} is the track title, {1} is the artist name
-                Title = String.Format (Catalog.GetString ("{0} by {1}"), 
+                Title = String.Format (Catalog.GetString ("{0} by {1}"),
                     track.DisplayTrackTitle, track.DisplayArtistName);
             } else {
                 Title = Catalog.GetString ("Banshee Media Player");
             }
-            
+
             OnTitleChanged ();
         }
 
@@ -186,8 +210,8 @@ namespace Banshee.Gui
 
             // This forces the toolbar to look like it's just a regular part
             // of the window since the stock toolbar look makes Banshee look ugly.
-            Style.ApplyDefaultBackground (toolbar.GdkWindow, true, State, 
-                args.Event.Area, toolbar.Allocation.X, toolbar.Allocation.Y, 
+            Style.ApplyDefaultBackground (toolbar.GdkWindow, true, State,
+                args.Event.Area, toolbar.Allocation.X, toolbar.Allocation.Y,
                 toolbar.Allocation.Width, toolbar.Allocation.Height);
 
             // Manually expose all the toolbar's children
